@@ -218,7 +218,13 @@ main (int argc, char **argv)
 	loop = g_main_loop_new (NULL, FALSE);
 
 	if (domain_ontology && domain_ontology_name) {
-		g_bus_watch_name_on_connection (connection, domain_ontology_name,
+		/* If we are running for a specific domain, we tie the lifetime of this
+		 * process to the domain. For example, if the domain name is
+		 * org.example.MyApp then this tracker-miner-rss process will exit as
+		 * soon as org.example.MyApp exits.
+		 */
+		domain_name = tracker_domain_ontology_get_domain (domain_ontology, NULL);
+		g_bus_watch_name_on_connection (connection, domain_name,
 		                                G_BUS_NAME_WATCHER_FLAGS_NONE,
 		                                NULL, on_domain_vanished,
 		                                loop, NULL);
